﻿/*==============================================================================*/
/*  Copyright (C) 2009-2018, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Ethernet Driver                                                  */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT10 Ver10.XX～　                                                */
/*  FileID :   21b                                                              */
/*  Version:   1.4                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Macro:  DT_ETHER_XXX                                                        */
/*  Desc:   Please select Ethernet mode                                         */
/*==============================================================================*/
//#define DT_ETHER_TCPIP_SERVER
//#define DT_ETHER_TCPIP_CLIENT
#define DT_ETHER_UDP

/*==============================================================================*/
/*  Please define DT_ADD_EVENT_INFO when you add Event Info to Test Point       */
/*==============================================================================*/
#define DT_ADD_EVENT_INFO

/*==============================================================================*/
/*  Please define DT_ADD_TIME_INFO when you add Time Value to Test Point        */
/*==============================================================================*/
#define DT_ADD_TIME_INFO

/*==============================================================================*/
/*  Please set define DT_USE_CRITICALSECTION when you use critical section.     */
/*==============================================================================*/
#define	DT_USE_CRITICALSECTION

using System;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.ComponentModel;
using System.Text;
using System.Threading;

public class DtTestPointDriver
{
#if DT_ADD_TIME_INFO
    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern bool QueryPerformanceFrequency(out long lpFrequency);
    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern bool QueryPerformanceCounter(out long lpPerformanceCount);
#endif

    /*=======================================================================*/
    /*  value Variables length. never set the value over 256                 */
    /*  Please do not change the value.                                      */
    /*=======================================================================*/
    private const int DT_MAX_VAL_SIZE = 256;

    /*=======================================================================*/
    /*  if DT_ADD_TIME_INFO is 1. Please set 8.                              */
    /*=======================================================================*/
#if DT_ADD_TIME_INFO
    private const int DT_TIME_INFO_SIZE = 8;
#else
    public const int DT_TIME_INFO_SIZE = 0;
#endif
    /*=======================================================================*/
    /*  if DT_ADD_EVENT_INFO is 1. Please set 4.                             */
    /*=======================================================================*/
#if DT_ADD_EVENT_INFO
    private const int DT_EVENT_INFO_SIZE = 4;
#else
    public const int DT_EVENT_INFO_SIZE = 0;
#endif

    /*=================================================================================================================*/
    /*  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 1	-> Normal TP with Event info and Time info. Please set 0x15.   */
    /*  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 0	-> Normal TP with Event info. Please set 0x05.                 */
    /*  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 1	-> Normal TP with Time info. Please set 0x11.                  */
    /*  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 0	-> Normal TP. Please set 0x01.                                 */
    /*=================================================================================================================*/
#if DT_ADD_EVENT_INFO && DT_ADD_TIME_INFO
    private const int DT_NRMLTP_ID = 0x15;
#elif (DT_ADD_EVENT_INFO) && (!DT_ADD_TIME_INFO)
    public const int DT_NRMLTP_ID = 0x05;
#elif (!DT_ADD_EVENT_INFO) && (DT_ADD_TIME_INFO)
    public const int DT_NRMLTP_ID = 0x11;
#else
    public const int DT_NRMLTP_ID = 0x01;
#endif
    /*=================================================================================================================*/
    /*  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 1	-> Variable TP with Event info and Time info. Please set 0x16. */
    /*  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 0	-> Variable TP with Event info. Please set 0x06.               */
    /*  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 1	-> Variable TP with Time info. Please set 0x12.                */
    /*  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 0	-> Variable TP. Please set 0x02.                               */
    /*=================================================================================================================*/
#if DT_ADD_EVENT_INFO && DT_ADD_TIME_INFO
    private const int DT_VALTP_ID = 0x16;
#elif (DT_ADD_EVENT_INFO) && (!DT_ADD_TIME_INFO)
    public const int DT_VALTP_ID = 0x06;
#elif (!DT_ADD_EVENT_INFO) && (DT_ADD_TIME_INFO)
    public const int DT_VALTP_ID = 0x12;
#else
    public const int DT_VALTP_ID = 0x02;
#endif

    /*=======================================================================*/
    /*  DynamicTracar TP size info.                                          */
    /*  Please do not change the value.                                      */
    /*=======================================================================*/
    private const int DT_HEADER_SIZE = 4;
    private const int DT_NRMLTP_SIZE = DT_HEADER_SIZE + 6 + DT_TIME_INFO_SIZE + DT_EVENT_INFO_SIZE;
    private const int DT_MAX_VALTP_SIZE = DT_NRMLTP_SIZE + 1 + DT_MAX_VAL_SIZE + DT_EVENT_INFO_SIZE;

    /*=======================================================================*/
    /*  DynamicTracer MaskCode                                               */
    /*=======================================================================*/
    static private string DT_IP_ADDRESS = "127.0.0.1";     /* customize */
    static private int DT_PORT = 3000;
    static private Socket socket = null;
    static private Socket soc = null;
#if DT_ETHER_TCPIP_CLIENT
    static public TcpClient client = null;
    static public NetworkStream stream = null;
#endif
    static private IPEndPoint dt_ep;
    static private byte[] buff = new byte[DT_NRMLTP_SIZE];
    static private byte[] val_buff = new byte[DT_MAX_VALTP_SIZE];
    static private object thisLock = new object();
    static private int dt_init = 0;
    static private int count = 0;
    static private int packet_size;
#if DT_USE_CRITICALSECTION
    static Mutex mut = new Mutex();
#endif

#if DT_ADD_TIME_INFO
    static private long DtStartCount;
    static private long DtEndCount;
    static private long frequency;
    static private long time_count = 0;

    /*=======================================================================*/
    /*  Add Time Info to TestPoint                                           */
    /*=======================================================================*/
    /* Normal TestPoint Method */
    static private void DtAddTimeInfo(byte[] time_buff)
    {
        QueryPerformanceCounter(out DtEndCount);
        double sec = (double)(DtEndCount - DtStartCount) / ((double)frequency); /* sec*/
        time_count = (long)(sec * 1000.0 * 1000.0);                             /* usec */
        time_buff[count++] = (byte)time_count;
        time_buff[count++] = (byte)(time_count >> 8);
        time_buff[count++] = (byte)(time_count >> 16);
        time_buff[count++] = (byte)(time_count >> 24);
        time_buff[count++] = (byte)(time_count >> 32);
        time_buff[count++] = (byte)(time_count >> 40);
        time_buff[count++] = (byte)(time_count >> 48);
        time_buff[count++] = (byte)(time_count >> 56);
    }
#endif

    /*=======================================================================*/
    /*  Add Event Info to TestPoint                                          */
    /*=======================================================================*/
    /* Normal TestPoint Method */
    static private void DtAddEventInfo(byte[] event_buff)
    {
        //            int nEventID = Process.GetCurrentProcess().Id;
        int nEventID = System.Threading.Thread.CurrentThread.ManagedThreadId;

        event_buff[count++] = (byte)nEventID;
        event_buff[count++] = (byte)(nEventID >> 8);
        event_buff[count++] = (byte)(nEventID >> 16);
        event_buff[count++] = (byte)(nEventID >> 24);
    }

    /*=======================================================================*/
    /*  TestPoint InsertCode Method                                          */
    /*=======================================================================*/
    /* Normal TestPoint Method */
    static public void DtTestPoint(int nBaseAddress, int nStepNumber)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        buff[count++] = (byte)0xFF;                   /* Header */
        buff[count++] = (byte)0xFF;                   /* Header */
        buff[count++] = (byte)DT_NRMLTP_SIZE;         /* Data Size */
        buff[count++] = (byte)0x00;                   /* Data Size */
        buff[count++] = (byte)DT_NRMLTP_ID;           /* Normal TP */
        buff[count++] = (byte)(nStepNumber);          /* nStepNumber */
        buff[count++] = (byte)(nStepNumber >> 8);     /* nStepNumber */
        buff[count++] = (byte)(nBaseAddress);         /* nBaseAddress */
        buff[count++] = (byte)(nBaseAddress >> 8);    /* nBaseAddress */
        buff[count++] = (byte)(nBaseAddress >> 16);   /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(buff);
#endif
        TP_DataSend(buff, count);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    /* MemoryOutput TestPoint Method */
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, byte theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 1;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)1;                        /* size */
        val_buff[count++] = (byte)theValue;                 /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, char theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 2;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)2;                        /* size */
        val_buff[count++] = (byte)theValue;                 /* value */
        val_buff[count++] = (byte)(theValue >> 8);          /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, short theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 2;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)2;                        /* size */
        val_buff[count++] = (byte)theValue;                 /* value */
        val_buff[count++] = (byte)(theValue >> 8);          /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, int theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 4;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)4;                        /* size */
        val_buff[count++] = (byte)theValue;                 /* value */
        val_buff[count++] = (byte)(theValue >> 8);          /* value */
        val_buff[count++] = (byte)(theValue >> 16);         /* value */
        val_buff[count++] = (byte)(theValue >> 24);         /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, long theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 8;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)8;                        /* size */
        val_buff[count++] = (byte)theValue;                 /* value */
        val_buff[count++] = (byte)(theValue >> 8);          /* value */
        val_buff[count++] = (byte)(theValue >> 16);         /* value */
        val_buff[count++] = (byte)(theValue >> 24);         /* value */
        val_buff[count++] = (byte)(theValue >> 32);         /* value */
        val_buff[count++] = (byte)(theValue >> 40);         /* value */
        val_buff[count++] = (byte)(theValue >> 48);         /* value */
        val_buff[count++] = (byte)(theValue >> 56);         /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, bool theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        count = 0;
        packet_size = DT_NRMLTP_SIZE + 1 + 1;

        if (dt_init == 0)
        {
            TpBusEthernetInit();
#if DT_ADD_TIME_INFO
            QueryPerformanceFrequency(out frequency);
            if (!QueryPerformanceCounter(out DtStartCount))
            {
                Console.WriteLine("QueryPerformanceCounter is out of support this system.");
                TpBusEthernetFinalize();
#if DT_USE_CRITICALSECTION
                mut.ReleaseMutex();
#endif
                return;
            }
#endif
        }

        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)0xFF;                     /* Header */
        val_buff[count++] = (byte)(packet_size);            /* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);     /* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;              /* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);            /* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);       /* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress);           /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);      /* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);     /* nBaseAddress */
#if DT_ADD_TIME_INFO
        DtAddTimeInfo(val_buff);
#endif
#if DT_ADD_EVENT_INFO
        DtAddEventInfo(val_buff);
#endif
        val_buff[count++] = (byte)1;                        /* size */
        val_buff[count++] = (byte)(theValue ? 0x01 : 0x00); /* value */
        TP_DataSend(val_buff, packet_size);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    /* open file */
    static private void TpBusEthernetInit()
    {
        try
        {
#if DT_ETHER_TCPIP_SERVER
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            dt_ep = new IPEndPoint(IPAddress.Any, DT_PORT);
            socket.Bind(dt_ep);
            socket.Listen(1);
            soc = socket.Accept();
#elif DT_ETHER_TCPIP_CLIENT
            client = new TcpClient(DT_IP_ADDRESS, DT_PORT);
            stream = client.GetStream();
#else
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPAddress ip = IPAddress.Parse(DT_IP_ADDRESS);
            dt_ep = new IPEndPoint(ip, DT_PORT);
#endif
            dt_init = 1;
        }
        catch (Exception)
        {
            dt_init = 0;
            Console.WriteLine("Error TpBusOutOpen()");
        }
    }

    /* close file */
    static private void TpBusEthernetFinalize()
    {
        try
        {
#if DT_ETHER_TCPIP_SERVER
            socket.Close();
            soc.Close();
#elif DT_ETHER_TCPIP_CLIENT
            stream.Close();
            client.Close();
#elif DT_ETHER_UDP
            socket.Close();
#endif
        }
        catch (Exception)
        {
            Console.WriteLine("Error TpBusOutClose()");
        }
    }

    /* MemoryOutput TestPoint Method (send 'val_buff') */
    static private void TP_DataSend(byte[] send_buff, int size)
    {
        try
        {
#if DT_ETHER_TCPIP_SERVER
            soc.Send(send_buff, size, SocketFlags.None);
#elif DT_ETHER_TCPIP_CLIENT
            stream.Write(send_buff, 0, size);
#elif DT_ETHER_UDP
            socket.SendTo(send_buff, 0, size, SocketFlags.None, dt_ep);
#endif
        }
        catch (Exception)
        {
        }
    }
}
